/* -------------------------------------------------------------------------*
 * Samples for the Attolic True Studio and the GCC                          *
 * Copyright (C) Lucjan Bryndza <lucjan.bryndza@ep.com.pl>  19.11.2010      *
 * Example No4 for the STM32 Butterfly2 and KAMODRGB                        *
 * Joystick change led saturation of the KAMODRGB                           *
 *                                                                          *
 * http://www.boff.pl                                                       *
 * -------------------------------------------------------------------------*/

#include "stm32f10x_lib.h"
#include "system.h"
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include "time.h"
#include "i2c_master.h"
/* ----------------------------------------------------------------------- */
//Joy configuration for the STM32 butterfly 1 and 2
#ifdef STM32_BUTTERFLY_1 /* Butterfly 1 */

#define JOY_PORT GPIOE
#define JOY_PORT_APB2 RCC_APB2Periph_GPIOE
#define JOY_PIN_L 12
#define JOY_PIN_R 11
#define JOY_PIN_D 10
#define JOY_PIN_U 9
#define JOY_PIN_OK 8

#else /* Butterfly 2 */

#define JOY_PORT GPIOC
#define JOY_PORT_APB2 RCC_APB2Periph_GPIOC
#define JOY_PIN_L 11
#define JOY_PIN_R 10
#define JOY_PIN_D 9
#define JOY_PIN_U 8
#define JOY_PIN_OK 7

#endif /*STM32_BUTTERFLY_1*/

#define JOY_BITS ((1<<JOY_PIN_L) | (1<<JOY_PIN_R) | (1<<JOY_PIN_D) | (1<<JOY_PIN_U) | (1<<JOY_PIN_OK))

//RGB I2c devaddr
#define RGBCTRL_I2CADDR 0x00

/* ----------------------------------------------------------------------- */

enum keys
{
   JOY_OK = 0x01,
   JOY_U =  0x02,
   JOY_D =  0x04,
   JOY_R =  0x08,
   JOY_L =  0x10
};

//Get joy keystate
static inline int joy_keys(void)
{
    return (~io_get_mask(JOY_PORT, JOY_BITS) & JOY_BITS) >> JOY_PIN_OK;
}


/* ------------------------------------------------------------------ */
#define RGB_INIT_NUMREG 3
#define RGB_INIT_NSEQ 2

//Initialize rgb controller
static int rgb_init(void)
{
    static const uint8_t r8[] = { 8, 255 };
    static const uint8_t r0_r1[] = { 0, 0, 0 };
    int ret = i2cm_transfer_7bit(RGBCTRL_I2CADDR, r0_r1, sizeof(r0_r1), NULL, 0);
    if( ret < 0) return ret;
    wait_ms(1);
    ret = i2cm_transfer_7bit(RGBCTRL_I2CADDR, r8,sizeof(r8), NULL, 0);
    return ret;
}

/* ------------------------------------------------------------------ */
#define RGBA(r,g,b,a) (unsigned)(r) | ((unsigned)(g)<<8) | ((unsigned)(b)<<16) | ((unsigned)(a)<<24)
#define RGB(r,g,b) RGBA(r,g,b,0)

static int rgb_set_color (unsigned c)
{
	uint8_t txbytes[] =
	{
		0x82,	   //Color register
		(c)>>24,   //A
		(c)>>16,   //B
		(c),	   //R
		(c)>>8 	   //G
	};
	return i2cm_transfer_7bit(RGBCTRL_I2CADDR,txbytes,sizeof(txbytes),NULL,0);
}
/* ----------------------------------------------------------------------- */
#define KBD_DEBOUNCE_TIME 10
#define I2C_SPEED 100000

int main(void)
{

   //JOY pin configuration input pullup VCC
    RCC->APB2ENR |= JOY_PORT_APB2;
    io_config_ext(JOY_PORT, JOY_BITS, GPIO_MODE_INPUT, GPIO_CNF_IN_PULLUP );
    io_set_clr_mask( JOY_PORT, JOY_BITS, JOY_BITS );

    //Initialize system timer
    time_init();
    //Initialize the i2c module
    i2cm_init(I2C_SPEED);
    //Initialize the RGB controler
    int res = rgb_init();
    if(res<0)
    {
        for(;;); //Doesn't work hangup
    }
    uint8_t r=0; uint8_t g=0; uint8_t b=0; uint8_t a=0;
    //Main loop
    for(;;)
    {
        //Read joy state
        int jstate1 = joy_keys();
        wait_ms(KBD_DEBOUNCE_TIME);
        int jstate2 = joy_keys();
        if(jstate1 && jstate1==jstate2)
        {
            switch(jstate1)
            {
                case JOY_U: r++; break;
                case JOY_D: g++; break;
                case JOY_L: b++; break;
                case JOY_R: a++; break;
                case JOY_OK: r=0; g=0; b=0; a=0; break;
            }
            rgb_set_color(RGBA(r,g,b,a));
        }
    }
    return 0;
}

/* ----------------------------------------------------------------------- */

